/*
 * Copyright 2010 Sergey Martynov http://studiomobile.ru
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.studiomobile.http.entity.nio;

import org.apache.http.entity.mime.*;

import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.Charset;
import java.util.List;
import java.util.Random;

class NMultipartEntityUtils {

    public static final char[] MULTIPART_CHARS = "1234567890abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ".toCharArray();
    public static final ByteBuffer TWO_DASHES = encode("--", MIME.DEFAULT_CHARSET);
    public static final ByteBuffer CR_LF = encode("\r\n", MIME.DEFAULT_CHARSET);
    public static final ByteBuffer FIELD_SEP = encode(": ", MIME.DEFAULT_CHARSET);

    public static String generateContentType(final String boundary, final Charset charset) {
        StringBuilder buffer = new StringBuilder();
        buffer.append("multipart/form-data; boundary=");
        buffer.append(boundary);
        if (charset != null) {
            buffer.append("; charset=");
            buffer.append(charset.name());
        }
        return buffer.toString();
    }

    public static String generateBoundary() {
        StringBuilder buffer = new StringBuilder();
        Random rand = new Random();
        int count = rand.nextInt(11) + 30; // a random size from 30 to 40
        for (int i = 0; i < count; i++) {
            buffer.append(MULTIPART_CHARS[rand.nextInt(MULTIPART_CHARS.length)]);
        }
        return buffer.toString();
    }

    public static ByteBuffer encode(final String string, final Charset charset) {
        return charset.encode(CharBuffer.wrap(string));
    }

    public static void encodeField(final MinimalField field, final Charset charset, final List<ByteBuffer> buffers)  {
        buffers.add(encode(field.getName(), charset));
        buffers.add(FIELD_SEP);
        buffers.add(encode(field.getBody(), charset));
        buffers.add(CR_LF);
    }

    public static void encodePartHeader(FormBodyPart part, HttpMultipartMode mode, Charset charset, List<ByteBuffer> buffers) {
        Header header = part.getHeader();
        switch (mode) {
            case STRICT:
                for (MinimalField field : header) {
                    encodeField(field, MIME.DEFAULT_CHARSET, buffers);
                }
                break;
            case BROWSER_COMPATIBLE:
                MinimalField cd = header.getField(MIME.CONTENT_DISPOSITION);
                encodeField(cd, charset, buffers);
                String filename = part.getBody().getFilename();
                if (filename != null) {
                    MinimalField ct = header.getField(MIME.CONTENT_TYPE);
                    encodeField(ct, charset, buffers);
                }
                break;
        }
        buffers.add(CR_LF);
    }
}
